# -*- coding: utf-8 -*-

"""
apiai
~~~~~~~~~~~~~~~~
This module provides a ApiAI classes to manage requests.
"""

try:  # Python 3
    from http.client import HTTPSConnection
except ImportError:
    from httplib import HTTPSConnection

import uuid

from .requests import (
    VoiceRequest,
    TextRequest,
    UserEntitiesRequest,
    events
)

import warnings

DEFAULT_VERSION = '20150910'


class ApiAI(object):
    """
        Main endpoint for using api.ai
        Provides request.

        Basic Usage::
            >>> ...
            >>> import apiai
            >>> ai = apiai.ApiAI(<CLIENT_ACCESS_TOKEN>)
            >>> text_request = ai.text_request()
            >>> ...

        :param client_access_token: client access token provided by https://api.ai/
        :type client_access_token: str or unicde
    """

    _connection_class = HTTPSConnection

    @property
    def client_access_token(self):
        """
            Client access token provided by https://api.ai/

            :rtype: str or unicode
        """

        return self._client_access_token

    @client_access_token.setter
    def client_access_token(self, client_access_token):
        """
            :type client_access_token: str or unicode
        """

        self._client_access_token = client_access_token

    @property
    def session_id(self):
        """
            session_id user for unique identifier of current application user.
            And it provide different contexts and entities for different users.
            Default it generated like uuid for every object of `ApiAI` class.


            :rtype: str or unicode
        """

        return self._session_id

    @session_id.setter
    def session_id(self, session_id):
        """
            :type session_id: str or unicode
        """

        self._session_id = session_id

    def __init__(self, client_access_token, session_id=None):
        super(ApiAI, self).__init__()
        self._client_access_token = client_access_token

        self._base_url = 'api.api.ai'
        self._version = DEFAULT_VERSION

        if session_id is None:
            self._session_id = uuid.uuid4().hex
        else:
            self._session_id = session_id

    def voice_request(self):
        """
            Construct a VoiceRequest, prepare it.
            Fields of request default filled from `ApiAI` parameters
            (session_id, version, client_access_token).

            This method is deprecated. Will be remove soon (1st feb 2017).
            The request working only for old paid account.

            Returns `VoiceRequest` object.

            :rtype VoiceRequest:
        """

        warnings.warn('This method is deprecated. Will be remove soon.', DeprecationWarning, stacklevel=2)

        request = VoiceRequest(
            self.client_access_token,
            self._base_url,
            self._version,
            self.session_id
        )

        return request

    def text_request(self):
        """
            Construct a `VoiceRequest`, prepare it.
            Fields of request default filled from `ApiAI` parameters
            (session_id, version,client_access_token).
            Returns `TextRequest` object.

            :rtype TextRequest:
        """

        request = TextRequest(
            self.client_access_token,
            self._base_url,
            self._version,
            self.session_id
        )

        return request

    def user_entities_request(self, user_entities=None):
        """
            Construct a `UserEntitiesRequest`, prepare it.
            Returns `UserEntitiesRequest` object.

            :rtype UserEntitiesRequest:
        """

        if user_entities is None:
            user_entities = []

        request = UserEntitiesRequest(
            self.client_access_token,
            self._base_url,
            user_entities
        )

        return request

    def event_request(self, event=None):
        """
            :type event: events.Event
            :rtype: events.EventRequest
        """

        request = events.EventRequest(
            self.client_access_token,
            self._base_url,
            self._version,
            self.session_id
        )

        request.event = event

        return request



# -*- coding: utf-8 -*-

from .query import QueryRequest
from .text import TextRequest
from .voice import VoiceRequest

from .query import Entry
from .query import Entity

from . import events

"""
apiai
~~~~~~~~~~~~~~~~
This module provides a ApiAI classes to manage requests.
"""

__all__ = [
    'QueryRequest',
    'TextRequest',
    'VoiceRequest',
    'Entry',
    'Entity',
    'events'
]



# -*- coding: utf-8 -*-

from .request import Request
from .query import QueryRequest
from .query import TextRequest
from .query import VoiceRequest
from .query import events

from .user_entities import UserEntitiesRequest

"""
apiai
~~~~~~~~~~~~~~~~
This module provides a ApiAI classes to manage requests.
"""

__all__ = [
    'Request',
    'QueryRequest',
    'TextRequest',
    'VoiceRequest',
    'UserEntitiesRequest',
    'EventRequest',
    'Event',
    'events'
]



# -*- coding: utf-8 -*-

from . import QueryRequest

import json


class TextRequest(QueryRequest):
    """
        TextRequest request class

        Send simple text queries.
        Query can be string or array of strings.
    """

    @property
    def query(self):
        """
            Query parameter, can be string or array of strings.
            Default equal None, nut your should fill this field before send
            request.

            :rtype: str or unicode
        """

        return self._query

    @query.setter
    def query(self, query):
        self._query = query

    def __init__(self, client_access_token, base_url, version, session_id):
        super(TextRequest, self).__init__(client_access_token,
                                          base_url,
                                          version,
                                          session_id)

        self.query = None

    def _prepare_headers(self):
        return {
            'Content-Type': 'application/json; charset=utf-8',
            'Content-Length': len(self._prepage_end_request_data()),
            'devMode': 'true'
        }

    def _prepage_begin_request_data(self):
        return None

    def _prepage_end_request_data(self):
        data = {
            'query': self.query,
            'lang': self.lang,
            'sessionId': self.session_id,
            'contexts': self.contexts,
            'timezone': self.time_zone,
            'resetContexts': self.resetContexts,
            'entities': self._prepare_entities(),
        }

        return json.dumps(data)



# -*- coding: utf-8 -*-

import numpy as np


class Resampler(object):
    def __init__(self,
                 source_samplerate,
                 destination_samplerate=16000,
                 dtype=np.int16,
                 channels=1):

        super(Resampler, self).__init__()
        self.ratio = destination_samplerate / float(source_samplerate)
        self.dtype = dtype
        self.channels = channels

    def resample(self, frame, frame_count):
        nparray = np.fromstring(frame, dtype=self.dtype)

        scale = self.ratio

        n = round(len(nparray) * scale)
        resampled = np.interp(
            np.linspace(0.0, 1.0, n, endpoint=False),
            np.linspace(0.0, 1.0, len(nparray), endpoint=False),
            nparray
        )

        return resampled.tolist(), resampled.astype(self.dtype).tostring()



# -*- coding: utf-8 -*-

from . import QueryRequest

import json


class Event(object):
    """
        Event model for EventRequest.
    """

    def __init__(self, name):
        super(Event, self).__init__()

        self._name = name
        self._data = None

    @property
    def name(self):
        """
            :rtype: str or unicode
        """
        return self._name

    @name.setter
    def name(self, name):
        self._name = name

    @property
    def data(self):
        """
            :rtype: dict
        """
        return self._data

    @data.setter
    def data(self, data):
        self._data = data


class EventRequest(QueryRequest):
    """
        EventRequest request class

        Send simple event queries.
    """

    def __init__(self, client_access_token, base_url, version, session_id):
        super(EventRequest, self).__init__(
            client_access_token,
            base_url,
            version,
            session_id
        )

        self._event = None

    @property
    def event(self):
        """
            :rtype: Event
        """
        return self._event

    @event.setter
    def event(self, event):
        self._event = event

    def _prepare_headers(self):
        return {
            'Content-Type': 'application/json; charset=utf-8',
            'Content-Length': len(self._prepage_end_request_data()),
            'devMode': 'true'
        }

    def _prepage_begin_request_data(self):
        return None

    def _prepage_end_request_data(self):
        event = {
            'name': self._event.name
        }

        if self._event.data:
            event['data'] = self._event.data

        data = {
            'event': event,
            'lang': self.lang,
            'sessionId': self.session_id,
            'contexts': self.contexts,
            'timezone': self.time_zone,
            'resetContexts': self.resetContexts,
            'entities': self._prepare_entities(),
        }

        return json.dumps(data)



# -*- coding: utf-8 -*-

import urllib
import os

try:  # Python 3
    from http.client import HTTPSConnection
except ImportError:
    from httplib import HTTPSConnection

try:
    import urllib.parse
except ImportError:
    import urllib


class Request(object):
    """Abstract request class
    Contain share information for all requests."""

    _connection_class = HTTPSConnection

    @property
    def client_access_token(self):
        """client access token provided by http://api.ai/"""
        return self._client_access_token

    @client_access_token.setter
    def client_access_token(self, client_access_token):
        self._client_access_token = client_access_token

    def __init__(self,
                 client_access_token,
                 base_url,
                 path,
                 query_parameters=[]
                 ):
        super(Request, self).__init__()

        self.base_url = base_url
        self.path = path
        self.query_parameters = query_parameters

        self.client_access_token = client_access_token

        self._prepare_proxy()
        self._prepare_request()

    def _prepare_entities(self):
        if self.entities:
            return list(map(lambda x: x._to_dict(), self.entities))
        return None

    def _prepare_proxy(self):

        self.proxy_enabled = False

        if os.environ.get("https_proxy"):
            self.proxy_enabled = True
            https_proxy = os.environ["https_proxy"]

            # As proxies are set like "export https_proxy=$http_proxy"
            # so it might start with 'https' or 'http'

            https_proxy = https_proxy.replace("https://", "").rstrip("/")
            https_proxy = https_proxy.replace("http://", "").rstrip("/")
            (self.proxy_host, self.proxy_port) = https_proxy.split(":")
            self.proxy_port = int(self.proxy_port)

    def _prepare_request(self, debug=False):
        if(self.proxy_enabled):
            self._connection = self._connection_class(
                self.proxy_host,
                self.proxy_port
            )

            self._connection.set_tunnel(self.base_url)
        else:
            self._connection = self._connection_class(self.base_url)

    def _connect(self):
        self._connection.connect()

        query = None

        try:
            query = urllib.urlencode(self.query_parameters)
        except AttributeError:
            query = urllib.parse.urlencode(self.query_parameters)

        full_path = self.path + '?' + query

        self._connection.putrequest('POST', full_path, skip_accept_encoding=1)

        headers = {
            'Accept': 'application/json',
            'Authorization': ('Bearer %s' % self.client_access_token)
        }

        headers.update(self._prepare_headers())

        for header_key, header_value in headers.items():
            self._connection.putheader(header_key, header_value)

        self._connection.endheaders()

        begin = self._prepage_begin_request_data()

        if begin is not None:
            self.send(begin.encode('utf-8'))

    def send(self, chunk):
        """Send a given data chunk of voice data."""

        if self._connection.sock is None:
            self._connect()

        self._connection.send(chunk)

    def _beforegetresponce(self):
        pass

    def getresponse(self):
        """Send all data and wait for response.
        """

        if self._connection.sock is None:
            self._connect()

        end = self._prepage_end_request_data()

        if end is not None:
            self.send(end.encode('utf-8'))

        self._beforegetresponce()

        return self._connection.getresponse()

    def _prepare_headers(self):
        raise NotImplementedError("Please Implement this method")

    def _prepage_begin_request_data(self):
        raise NotImplementedError("Please Implement this method")

    def _prepage_end_request_data(self):
        raise NotImplementedError("Please Implement this method")



# -*- coding: utf-8 -*-

from .user_entities_request import UserEntity
from .user_entities_request import UserEntityEntry
from .user_entities_request import UserEntitiesRequest

"""
apiai
~~~~~~~~~~~~~~~~
This module provides a ApiAI classes to manage requests.
"""

__all__ = [
    'UserEntity',
    'UserEntityEntry',
    'UserEntitiesRequest'
]



# -*- coding: utf-8 -*-

"""
apiai
~~~~~~~~~~~~~~~~
This module provides a ApiAI classes to manage requests.
"""

from .requests.query import Entry
from .requests.query import Entity

from .requests import Request
from .requests import QueryRequest
from .requests import events

from .apiai import ApiAI
from .apiai import TextRequest
from .apiai import VoiceRequest
from .apiai import UserEntitiesRequest

from .requests.user_entities import UserEntity
from .requests.user_entities import UserEntityEntry

from .VAD import VAD
from .resampler import Resampler

__author__ = "Dmitriy Kuragin"
__copyright__ = "Copyright 2016 api.ai"
__contributors__ = []
__license__ = "Apache 2.0"
__version__ = "1.2.2"

__all__ = [
    'apiai',
    'ApiAI',
    'Request',
    'Entry',
    'Entity',
    'QueryRequest',
    'TextRequest',
    'VoiceRequest',
    'UserEntitiesRequest',
    'UserEntity',
    'UserEntityEntry',
    'VAD',
    'Resampler',
    'events'
]



# -*- coding: utf-8 -*-

from ..request import Request

from ..query import Entity
from ..query import Entry

import json


class UserEntity(Entity):
    """UserEntity object used for upload user entities.
    Detail information about user entities you can see at our site
    https://docs.api.ai/docs/userentities"""

    @property
    def session_id(self):
        """session_id parameter used for determinate of every unique users."""
        return self._session_id

    @session_id.setter
    def session_id(self, session_id):
        self._session_id = session_id

    @property
    def extend(self):
        return self._extend

    @extend.setter
    def extend(self, extend):
        """extend parameter used definition user entities logic. If True then
        uploaded user entities will be mixed with user entities specified in
        server side else currently uploaded entities witll uverride
        server entities."""
        self._extend = extend

    def __init__(self, name, entries, session_id=None, extend=False):
        super(UserEntity, self).__init__(name, entries)

        self._session_id = session_id
        self._extend = extend

    """Private method used for object serialization."""
    def _to_dict(self):
        parent_data = super(UserEntity, self)._to_dict()
        if self._session_id is not None:
            parent_data['sessionId'] = self._session_id

        parent_data['extend'] = self._extend

        return parent_data


class UserEntityEntry(Entry):
    """UserEntityEntry object used for upload user entities.
    Detail information about user entities you can see at our site
    https://docs.api.ai/docs/userentities"""

    pass


class UserEntitiesRequest(Request):
    """UserEntitiesRequest is request for upload user entities.
    Detail see http://docs.api.ai/"""

    @property
    def user_entities(self):
        "user_entities parameter for specification of same user entities."
        return self._user_entities

    @user_entities.setter
    def user_entities(self, user_entities):
        self._user_entities = user_entities

    def __init__(self, client_access_token, base_url, user_entities=[]):
        super(UserEntitiesRequest, self).__init__(client_access_token,
                                                  base_url,
                                                  '/v1/userEntities',
                                                  {})

        self._user_entities = user_entities

    def _prepare_headers(self):
        return {
            'Content-Type': 'application/json; charset=utf-8',
            'Content-Length': len(self._prepage_end_request_data())
        }

    def _prepage_begin_request_data(self):
        return None

    def _prepage_end_request_data(self):
        return json.dumps(list(map(lambda x: x._to_dict(), self._user_entities)))



# -*- coding: utf-8 -*-

from ..request import Request

from time import gmtime
from time import strftime


class _Serializable(object):
    """
        Abstract serializable class.
        All classes implemended this used for request parameters.
        It can be serializable to JSON values for request parameters.
    """

    def _to_dict(self):
        """
            Private method used for object serialization.
        """

        raise NotImplementedError()


class Entry(_Serializable):
    """
        User entry for class `Entity`
        Entry objects, which contain reference names and synonyms for `Entity`.
        For detail information about entries see
        https://docs.api.ai/v6/docs/concept-entities
    """

    @property
    def value(self):
        """
            Entry's value A canonical name to be used in place of the synonyms.
            Example: `New York`

            :rtype: str or unicode
        """

        return self._value

    @value.setter
    def value(self, value):
        """
            :type value: str or unicode
        """

        self._value = value

    @property
    def synonyms(self):
        """
            The array of synonyms.
            Example: `["New York", "@big Apple",
            "city that @{never, seldom, rarely} sleeps"]`

            :rtype: list of (str or unicode)
        """

        return self._synonyms

    @synonyms.setter
    def synonyms(self, synonyms):
        """
            :type synonyms: list of (str or unicode)
        """
        self._synonyms = synonyms

    def __init__(self, value, synonyms):
        """Construct a `Entry` and fill default values."""
        super(Entry, self).__init__()

        self._value = value
        self._synonyms = synonyms

    """Private method used for object serialization."""
    def _to_dict(self):
        return {
            'value': self.value,
            'synonyms': self.synonyms
        }


class Entity(_Serializable):
    """
        User entity for `Request`
        `Entity` is used to create, retrieve and update user-defined entity
        objects. For detail information about entities see
        https://docs.api.ai/v6/docs/concept-entities
    """

    @property
    def name(self):
        """
            Entity name.

            :rtype: str or unicode
        """

        return self._name

    @name.setter
    def name(self, name):
        """
            :type name: str or unicode
        """

        self._name = name

    @property
    def entries(self):
        """
            Entity entries. Array of `Entry` class objects

            :rtype: list of Entry
        """

        return self._entries

    @entries.setter
    def entries(self, entries):
        """
            :type entries: list of Entry
        """

        self._entries = entries

    def __init__(self, name, entries):
        super(Entity, self).__init__()

        self.name = name
        self.entries = entries

    def _to_dict(self):
        """
            Private method used for object serialization.
        """

        return {
            'name': self.name,
            'entries': list(map(lambda x: x._to_dict(), self.entries))
        }


class QueryRequest(Request):
    """
        Abstract request class
        Contain share information for all query requests.
    """

    @property
    def lang(self):
        """
            lang property used for server determination current request language.
            In `VoiceRequest` used for determinate language for ASR
            (Speech Recognitions) service. Default equal 'en'. For detail
            information about support language see
            https://docs.api.ai/docs/languages

            :rtype: str or unicode

        """

        return self._lang

    @lang.setter
    def lang(self, lang):
        self._lang = lang

    @property
    def resetContexts(self):
        """
            resetContexts used for reset (cancel/disable) all previous all contexts.
            All contexts provided in current request will be setted after reset.
            Default equal False.

            :rtype: bool
        """
        return self._resetContexts

    @resetContexts.setter
    def resetContexts(self, resetContexts):
        self._resetContexts = resetContexts

    @property
    def contexts(self):
        """
            Array of context objects. for detail information see
            https://docs.api.ai/v6/docs/concept-contexts

            :rtype: list of dict
        """
        return self._contexts

    @contexts.setter
    def contexts(self, contexts):
        self._contexts = contexts

    @property
    def session_id(self):
        """
            session_id user for unique identifier of current application user.
            And it provide different contexts and entities for different users.

            :rtype: str or unicode
        """
        return self._session_id

    @session_id.setter
    def session_id(self, session_id):
        self._session_id = session_id

    @property
    def time_zone(self):
        """
            Time zone from IANA Time Zone Database
            (see http://www.iana.org/time-zones). Examples: `America/New_York`,
            `Europe/Paris`. Time zone used for provide information about time and
            other parameters depended by time zone.
            Default equal `strftime("%z", gmtime())` -> used current system time
            zone.

            :rtype: str or unicode
        """

        return self._time_zone

    @time_zone.setter
    def time_zone(self, time_zone):
        self._time_zone = time_zone

    @property
    def entities(self):
        """Array of entities that replace developer defined entities for this
        request only.
        The entity(ies) need to exist in the developer console."""
        return self._entities

    @entities.setter
    def entities(self, entities):
        self._entities = entities

    def __init__(self, client_access_token, base_url, version, session_id):
        super(QueryRequest, self).__init__(client_access_token,
                                           base_url,
                                           '/v1/query',
                                           {'v': version}
                                           )

        self.lang = 'en'
        self.resetContexts = False
        self.contexts = []
        self.entities = None

        self.version = version
        self.session_id = session_id

        self.time_zone = strftime("%z", gmtime())

    def _prepare_entities(self):
        if self.entities:
            return list(map(lambda x: x._to_dict(), self.entities))
        return None

    def _prepare_headers(self):
        raise NotImplementedError("Please Implement this method")

    def _prepage_begin_request_data(self):
        raise NotImplementedError("Please Implement this method")

    def _prepage_end_request_data(self):
        raise NotImplementedError("Please Implement this method")



# -*- coding: utf-8 -*-

from . import QueryRequest

import sys
import uuid
import json
import warnings

PY3 = sys.version_info[0] == 3

if PY3:
    def b(s):
        return s.encode("latin-1")
else:
    def b(s):
        return s


class VoiceRequest(QueryRequest):
    """
        .. deprecated:: 1.0.3

        VoiceRequest request class

        Send voice data by chunks.

        This is method deprecated. Will be removed on 1 Feb 2016.
        THis is method working only for for old paid plans and
        doesn't work for all new users.

        Basic Usage::
            >>> ...
            >>> voice_request = ai.text_request()
            >>> bytessize = 2048
            >>>
            >>> with open('log.raw', 'rb') as f:
            >>>     data = f.read(bytessize)
            >>>     while data:
            >>>         request.send(data)
            >>>         data = f.read(bytessize)
            >>>
            >>> request.getresponse()
            <JSON response>
    """

    def __init__(self, client_access_token, base_url, version, session_id):
        warnings.warn('VoiceRequest will be removed on 1 Feb 2016', DeprecationWarning)

        super(VoiceRequest, self).__init__(
            client_access_token,
            base_url,
            version,
            session_id
        )

        self._audio_mime_type = None

    @property
    def audio_mime_type(self):
        """
            audio mime type, default value equal None. If value is None then
            uses 'audio/wav' mime type.

            :rtype: str or unicode
        """

        return self._audio_mime_type

    @audio_mime_type.setter
    def audio_mime_type(self, value):
        """
            audio mime type setter
            :type value: str or unicode
        """

        self._audio_mime_type = value

    def send(self, chunk):
        """
            Send a given data chunk of voice data.
        """

        parts = []

        parts.append('%x' % len(chunk))

        if PY3:
            parts.append(chunk.decode('latin-1'))
        else:
            parts.append(chunk)

        parts.append('')

        newChunk = '\r\n'.join(parts)

        super(VoiceRequest, self).send(b(newChunk))

    def _prepare_headers(self):
        uuid_hex = uuid.uuid4().hex
        self.boundary = ('--------{0}'.format(uuid_hex)).encode('utf-8')

        return {
            'Content-Type': 'multipart/form-data; boundary=%s' % self.boundary,
            'Transfer-Encoding': 'chunked',
            'Connection': 'keep-alive',
        }

    def _prepage_begin_request_data(self):
        data = '--%s\r\n' % self.boundary
        data += 'Content-Disposition: form-data; name="request"\r\n'
        data += 'Content-Type: application/json\r\n\r\n'

        data += json.dumps(
            {
                'lang': self.lang or 'en',
                'sessionId': self.session_id,
                'contexts': self.contexts,
                'timezone': self.time_zone,
                'resetContexts': self.resetContexts,
                'entities': self._prepare_entities()
            }
        )

        data += '\r\n'
        data += '--%s\r\n' % self.boundary
        data += 'Content-Disposition: form-data; name="voiceData"\r\n'
        data += 'Content-Type: %s\r\n\r\n' % (self._audio_mime_type_prepare())

        return data

    def _audio_mime_type_prepare(self):
        current_audio_mime_type = self.audio_mime_type
        if current_audio_mime_type is None:
            current_audio_mime_type = 'audio/wav'

        return current_audio_mime_type

    def _prepage_end_request_data(self):
        return '\r\n--%s--\r\n' % self.boundary

    def _beforegetresponce(self):
        self._connection.send(b('0\r\n\r\n'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    Setup script for using and create package for pip.
"""

import sys
import os

# import apiai

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    # os.system('python setup.py sdist upload')
    os.system('python setup.py sdist bdist_wheel upload')
    sys.exit()

PACKAGES = [
    'apiai',
    'apiai.requests',
    'apiai.requests.query',
    'apiai.requests.user_entities'
]

REQUIRES = [
    'numpy'
]

EXTRAS_REQUIRE = {
    'numpy': ['numpy']
}

with open('README.rst', 'r') as f:
    README = f.read()

with open('HISTORY.rst', 'r') as f:
    HISTORY = f.read()

setup(
    name='apiai',
    version='1.2.2',
    description=(
        'The API.AI iOS SDK makes it easy to integrate speech '
        'recognition with API.AI natural language processing '
        'API on iOS devices.'
    ),
    long_description=README + '\n\n' + HISTORY,
    author='Dmitriy Kuragin',
    author_email='kuragin@speaktoit.com',
    license='Apache 2.0',
    url='https://api.ai/',
    packages=PACKAGES,
    install_requires=REQUIRES,
    # extras_require=EXTRAS_REQUIRE,
    package_data={'': ['LICENSE']},
    classifiers=(
        # 'Development Status :: 5 - Production/Stable',
        # 'Intended Audience :: Developers',
        # 'Natural Language :: English',
        # 'License :: OSI Approved :: Apache Software License',
        # 'Programming Language :: Python',
        # 'Programming Language :: Python :: 2.7',
        # 'Programming Language :: Python :: 3',
        # 'Programming Language :: Python :: 3.3',
        # 'Programming Language :: Python :: 3.4'
    ),
)
